# adaptation of Yuzu Nintendo Switch Emulator (https://github.com/yuzu-emu/yuzu) unswizzling by https://github.com/orgs/yuzu-emu/people
# https://git.metaa.io/mirror/yuzu/blob/a3e82e8e1f5cb39246f30cac045db8e243f0daee/src/video_core/textures/decoders.cpp
# original source: Tegra K1 Technical Reference Manual, p.1187-1188 (https://developer.nvidia.com/embedded/tegra-k1-reference)
# assume GPLv2+ License: https://github.com/yuzu-emu/yuzu/blob/master/license.txt

def imageUnswizzleSwitch(buffer, width, height, bpb, block_height=16, width_pad=16, height_pad=16):
    # setup
    out = bytearray(len(buffer))
    if width % width_pad or height % height_pad:
        width_show = width
        height_show = height
        width = width_real = ((width + width_pad - 1) // width_pad) * width_pad
        height = height_real = ((height + height_pad - 1) // height_pad) * height_pad
    else:
        width_show = width_real = width
        height_show = height_real = height
    image_width_in_gobs = width * bpb // 64

    # unswizzling
    for Y in range(height):
        for X in range(width):
            Z = Y * width + X
            GOB_address = 0 + (Y // (8 * block_height)) * 512 * block_height * image_width_in_gobs + (X * bpb // 64) * 512 * block_height + (Y % (8 * block_height) // 8) * 512
            X *= bpb
            address = GOB_address + ((X % 64) // 32) * 256 + ((Y % 8) // 2) * 64 + ((X % 32) // 16) * 32 + (Y % 2) * 16 + (X % 16)
            out[Z * bpb:(Z + 1) * bpb] = buffer[address:address + bpb]

    # crop
    if width_show != width_real or height_show != height_real:
        crop = bytearray(width_show * height_show * bpb)
        for Y in range(height_show):
            OffsetIn = Y * width_real * bpb
            OffsetOut = Y * width_show * bpb
            crop[OffsetOut:OffsetOut + width_show * bpb] = out[OffsetIn:OffsetIn + width_show * bpb]
        out = crop

    return out